// Copyright 2006-2007 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz
// or its licensors, as applicable.
//
// You may not use this file except under the terms of the accompanying license.
//
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Project: 
// File: 
// Purpose: 
// Responsible: tmbdev
// Reviewer:
// Primary Repository:
// Web Sites: www.iupr.org, www.dfki.de

#include <stdio.h>
#include <float.h>
#include "fst/lib/fstlib.h"
#undef CHECK
#include "colib.h"
#include "fstbuilder.h"
#include "fstutil.h"

using namespace fst;
using namespace colib;

namespace ocropus {
    class FstBuilderImpl : public FstBuilder {
        autoref<StdVectorFst> fst;
        int nextstate;
        int maxstate;
    public:
        FstBuilderImpl() {
            clear();
        }
        const char *description() {
            return "FstBuilder";
        }
        void clear() {
            fst = 0;
            nextstate = 0;
            maxstate = 0;
        }
        int newState() {
            int result = fst->AddState();
            // Check that states are >=0 (we rely on that).
            CHECK_ARG(result>=0);
            // Check that states are assigned contiguously.
            // The library documentation seems to imply that they are.
            // Knowing this makes our assertions about states sharper.
            // Also, it is important for being able to use states as array indexes.
            CHECK_ARG(result==nextstate++);
            if(result>maxstate) maxstate = result;
            return result;
        }
        void addTransition(int from,int to,nuchar c,float cost,int id) {
            CHECK_ARG(from>=0 && from<=maxstate);
            CHECK_ARG(to>=0 && to<=maxstate);
            CHECK_ARG(id>-(1<<30) && id<(1<<30));
            id &= 0x7fffffff;
            CHECK_ARG(c.value>-(1<<30) && c.value<(1<<30));
            CHECK_ARG(cost>-1e10 && cost<1e10 && cost==cost);
            c.value &= 0x7fffffff;
            // fprintf(stderr,"* from %d id %d c %d cost %f to %d\n",from,id,c.ord(),cost,to);
            fst->AddArc(from,StdArc(id,c.ord(),cost,to));
        }
        void setStart(int i) {
            CHECK_ARG(i>=0 && i<=maxstate);
            fst->SetStart(i);
        }
        void setAccept(int i,float f) {
            CHECK_ARG(i>=0 && i<=maxstate);
            fst->SetFinal(i,f);
        }
        int special(const char *s) {
            return 0;
        }
        void bestpath(nustring &result) {
            floatarray costs;
            intarray ids;
            ocropus::bestpath(result,costs,ids,*fst);
        }
        StdVectorFst *take() {
            // if(!Verify(*fst)) throw "bad fst or bug in fst library";
            return fst.move();
        }
    };

    FstBuilder *make_FstBuilder() {
        return new FstBuilderImpl();
    }
} //namespace
