// Copyright 2006-2007 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz 
// or its licensors, as applicable.
// 
// You may not use this file except under the terms of the accompanying license.
// 
// Licensed under the Apache License, Version 2.0 (the "License"); you
// may not use this file except in compliance with the License. You may
// obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Project:
// File: 
// Purpose: 
// Responsible: tmb
// Reviewer: 
// Primary Repository: 
// Web Sites: 

#include <stdio.h>
#include <wchar.h>
#include <stdlib.h>
#include "colib.h"
#include "triv-hocr.h" 
#include "ocr-utils.h" 

using namespace colib;
using namespace ocropus;

namespace ocropus {
    param_bool remove_hyphens("remove_hyphens",true,
            "join words separated by a hyphen and a newline");

    void output_hOCR_header(FILE *f) {
        fprintf(f, "<!DOCTYPE html\n");
        fprintf(f, "     PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n");
        fprintf(f, "     \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
        fprintf(f, "<html  xmlns=\"http://www.w3.org/1999/xhtml\">\n");
        fprintf(f, " <head>\n");
        fprintf(f, "  <meta name='ocr-system' content='OCRopus %s' />\n", get_version_string());  
        fprintf(f, "  <meta name='ocr-capabilities' content='ocr_line ocr_page' />\n");
        fprintf(f, "  <meta name='ocr-langs' content='en' />\n");
        fprintf(f, "  <meta name='ocr-scripts' content='Latn' />\n");
        fprintf(f, "  <meta name='ocr-microformats' content='' />\n");
        fprintf(f, "  <title>OCR Output</title>\n");
        fprintf(f, " </head>\n");
        fprintf(f, "<body>\n");
    }
    
    void output_hOCR_footer(FILE *f) {
        fprintf(f, "</body>\n");
        fprintf(f, "</html>\n");
    }
    
    void output_hOCR(FILE *f, RecognizedPage &page) {
        fprintf(f, "<div class='ocr_page' title='image %s'>\n", page.description());
        bool paragraph = false;
        for(int line = 0; line < page.linesCount(); line++) {
            if(page.isParagraph(line)) {
                if(paragraph)
                    fprintf(f, "</p>\n");
                fprintf(f, "<p class='ocr_par'>\n");
                paragraph = true;
            }
            nustring text;
            page.text(text, line);
            rectangle bbox = page.bbox(line);
            if(!page.isHeader(line)) {
                output_hOCR_line(f, text, bbox, page.height());
            } else {
                if(paragraph)
                    fprintf(f, "</p>\n");
                if(line == 0 || !page.isHeader(line-1))
                    fprintf(f, "<h3>"); 
                output_hOCR_line(f, text, bbox, page.height());
                if(line == page.linesCount() - 1 || !page.isHeader(line + 1))
                    fprintf(f, "</h3>\n");
                paragraph = false;
            }
        }
        if(paragraph) {
            // -- close final paragraph --
            fprintf(f, "</p>\n");
        }
        fprintf(f, "</div>\n");
        if(page.timeReport())
            fprintf(f,"<!-- %s -->\n", page.timeReport());
    }

    void output_hOCR(FILE *f, narray<RecognizedPage> &pages) {
        output_hOCR_header(f);
        for(int i = 0; i < pages.length(); i++)
            output_hOCR(f, pages[i]);
        output_hOCR_footer(f);        
    }

    void output_hOCR_char(FILE *file, int c) {
        switch(c) {
            case '&': fprintf(file, "%ls", L"&amp;"); break;
            case '<': fprintf(file, "%ls", L"&lt;"); break;
            case '>': fprintf(file, "%ls", L"&gt;"); break;
            default:
                if(c < 128)
                    fprintf(file, "%c", c);
                else
                    fprintf(file, "&#%d;", c); // unicode escape
        }
    }

    void output_hOCR_line(FILE *file, nustring &line, rectangle &bbox, int page_height) {
        if(!line.length()) return;
        fprintf(file, "<span class='ocr_line' title='bbox %d %d %d %d'>",
                       bbox.x0,                    // left
                       page_height - bbox.y1,  // top
                       bbox.x1,                    // right
                       page_height - bbox.y0); // bottom
        
        for(int i = 0; i < line.length()-1; i++) {
            output_hOCR_char(file, line[i].ord());
        }
        if (remove_hyphens && line.length() && line[line.length() - 1].ord() == '-') {
            fprintf(file, "</span>");
        } else {
            if (line.length())
                output_hOCR_char(file, line[line.length()-1].ord());
            fprintf(file, "</span>\n");
        }
    }

}
